package jcop.compiler;

import java.io.BufferedInputStream;
import java.io.BufferedOutputStream;
import java.io.File;
import java.io.FileInputStream;
import java.io.FileOutputStream;
import java.io.IOException;
import java.io.InputStream;
import java.io.OutputStream;
import java.net.URISyntaxException;
import java.util.jar.JarFile;
import java.util.zip.ZipEntry;

import jcop.Globals;
import jcop.JCopAspect;
import jcop.Tools;
import jcop.generation.AspectSourceCodeGeneration;
import AST.Program;

public class AuxiliaryCompilerFilesHandler {
	private CompilerMessageStream msg;
	
	private final String[] files = new String[]{
		Globals.Files.concreteLayer,
		Globals.Files.partialMethod,
		Globals.Files.layer,
		Globals.Files.layerd,
		Globals.Files.composition,
		Globals.Files.layeredState};
	

	public AuxiliaryCompilerFilesHandler(CompilerMessageStream msg) {
		this.msg = msg;
		File jcopFolder = createJCopFolder();
		initProgram(jcopFolder);
	}

	private void initProgram(File jcopFolder) {
		String rootDir = getWorkingDir();
		
		removeJCopSpecificClassFiles();
		for (String file : files) 
			copyLayerClassInFolder(jcopFolder, file);		
	}

	public void writeJCopAspectToDisc(boolean debugMode) {
		new AspectSourceCodeGeneration(getWorkingDir() + File.separator + 
									   Globals.jcopFolder)
				.writeToFile(JCopAspect.getInstance());
		
		if (debugMode)
			new AspectSourceCodeGeneration(getWorkingDir() + File.separator + 
										   "compiled-sources" + File.separator + 
										   Globals.jcopFolder)
				.writeToFile(JCopAspect.getInstance());
	}

	public File createJCopFolder() {
		String rootDir = getWorkingDir();
		String folderName = rootDir + File.separator + Globals.jcopFolder
				+ File.separatorChar;

		File jcopFolder = new File(folderName);
		if (!jcopFolder.exists())
			jcopFolder.mkdirs();
		return jcopFolder;
	}

	public void cleanProgram() {
		for (String file : files) 
			new File (Globals.jcopFolder, file).delete();
	}
	
	public void removeJCopSpecificClassFiles() {
		Tools.deleteClassfilesInDir(new File(getWorkingDir() + File.separator));
	}

	public String getWorkingDir() {
		String rootDir = System.getProperty("user.dir");
		if (Program.hasValueForOption("-rootDir"))
			rootDir = Program.getValueForOption("-rootDir");
		return rootDir;
	}

	private InputStream getFileInputStream(String fileName) {
		try {
			File sourceLayerFile = new File(fileName);
			return new FileInputStream(sourceLayerFile);
		} catch (IOException e) {
			e.printStackTrace();
		}
		return null;
	}

	public void copyLayerClassInFolder(File folder, String fileName) {
		try {
			File targetLayerFile = new File(folder, fileName);
			InputStream is = getJarInputStream(fileName);
			if (is == null)
				is = getFileInputStream(fileName);
			copyToFile(is, targetLayerFile);
			is.close();
		} catch (Exception e) {
			e.printStackTrace();
		}
	}

	private InputStream getJarInputStream(String fileName) {
		try {
			JarFile jar = getJarFile();
			ZipEntry zippedLayerClass = jar.getEntry(fileName);
			return jar.getInputStream(zippedLayerClass);
		} catch (Exception e) {
		}
		return null;
	}

	private JarFile getJarFile() throws URISyntaxException, IOException {
		File file = new File(this.getClass().getProtectionDomain()
				.getCodeSource().getLocation().toURI());
		return new JarFile(file.getAbsolutePath());
	}

	private void copyToFile(InputStream is, File outputFile) {
		try {
			InputStream in = new BufferedInputStream(is);
			OutputStream out = new BufferedOutputStream(new FileOutputStream(
					outputFile));
			byte[] buffer = new byte[2048];
			int nBytes;
			while ((nBytes = in.read(buffer)) > 0)
				out.write(buffer, 0, nBytes);
			out.flush();
			out.close();
		} catch (IOException e) {
			msg.println("cannot create " + outputFile.getAbsolutePath());
			e.printStackTrace();
		}
	}
}
